/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */

package net.craftforge.essential.context.server;

import com.sun.net.httpserver.HttpExchange;
import net.craftforge.essential.controller.Request;
import net.craftforge.essential.controller.utils.CaseInsensitiveMap;
import net.craftforge.essential.controller.utils.ControllerUtils;

import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * A lightweight HTTP server implementation of a controller request.
 *
 * @author Christian Bick
 * @since 19.06.11
 */
public class HttpServerRequestImpl implements Request {

    private String context;
    private HttpExchange httpExchange;

    /**
     * Creates an HTTP server request from a context an HttpExchange.
     *
     * @param context The context
     * @param httpExchange The HTTP exchange
     */
    public HttpServerRequestImpl(String context, HttpExchange httpExchange) {
        this.context = ControllerUtils.standardUri(context);
        this.httpExchange = httpExchange;
    }

    /**
     * {@inheritDoc}
     */
    public String getHttpMethod() {
        return httpExchange.getRequestMethod();
    }

    /**
     * {@inheritDoc}
     */
    public String getUrlInfoPart() {
        return httpExchange.getRequestURI().getPath().replaceFirst(context, "");
    }

    /**
     * {@inheritDoc}
     */
    public Map<String, String> getHeaders() {
        Map<String, String> headers = new CaseInsensitiveMap<String>();
        for (String headerName : httpExchange.getRequestHeaders().keySet()) {
            List<String> headerValues = httpExchange.getRequestHeaders().get(headerName);
            if (headerValues.size() > 0) {
                headers.put(headerName, headerValues.get(0));
            }
        }
        return headers;
    }

    /**
     * {@inheritDoc}
     */
    public InputStream getBodyInputStream() {
        return httpExchange.getRequestBody();
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Map<String, String[]> getParameters() {
        String uriQueryPart = httpExchange.getRequestURI().getQuery();
        if (uriQueryPart == null || uriQueryPart.isEmpty()) {
            return Collections.EMPTY_MAP;
        }
        try {
            return ControllerUtils.getUrlParameters("UTF-8", uriQueryPart);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Cannot decode URL query string: Unsupported URL encoding", e);
        }
    }
}
